##############################################################################
## Program      : rsdialog.tcl
## Version      : 1.10
##
## Title        : Message box function
##
## MODIF ARI    : 12/09/96 : correction
##                in function winquestion(),
##                   defbutton[1-3]  replaced by defbutton([0-9]+)
##                   dangbutton[1-3] replaced by dangbutton([0-9]+)
## FIN MODIF
###############################################################################

#####################################
# Inialisation of Globals Variables #
#####################################
set dataMsgbox(0) 0
set returnCode 0

if ![info exists fgldir] {
  set fgldir /wtkclt
  catch {set fgldir $env(FGLDIR)}
}

if { ! $fglWtkClient } {
  set fglicon $fgldir/bmp
  proc myMsgbox { mess title options pid } {
    winquestion "$mess" "$title" "$options" $pid
  }
 } else {
  set fglicon $wtk_clientdir/../bmp
  proc myMsgbox { mess title options items pid } {
    global dpyChannel
    set dc $dpyChannel
    update
    if { [catch {sockCheck $dc}] } {
      sockSelect $dc
    }
    set ret [msgbox 0 "$mess" "$title" "$options"]
    if { [catch {sockCheck $dc}] } {
      sockSelect $dc
    }
    if { [llength $items] == 0 } {
      fglSendKey $dc F25
    } else {
      set no 0
      foreach i $items {
        if { $i == $ret } {
          fglSendKey $dc F[expr $no + 25]
        }
        incr no
      }
    }
  }
}
option add *FglTopMsgbox.background     black              userDefault
option add *FglTopMsgbox.relief         flat               userDefault
option add *FglTopMsgbox.borderWidth    1                  userDefault
option add *FglTitleMsgbox.background   #00008b            userDefault
option add *FglTitleMsgbox.foreground   white              userDefault
option add *FglTitleMsgbox.relief       raised             userDefault
option add *FglTitleMsgbox.borderWidth  1                  userDefault
option add *FglFrameMsgbox.background   $config(defaultBg) userDefault
option add *FglFrameMsgbox.relief       raised             userDefault
option add *FglFrameMsgbox.borderWidth  1                  userDefault
option add *FglMessMsgbox.background    $config(defaultBg) userDefault
option add *FglMessMsgbox.borderWidth   0                  userDefault
option add *FglMessMsgbox.relief        flat               userDefault
option add *FglBitmapMsgbox.background  $config(defaultBg) userDefault
option add *FglBitmapMsgbox.borderWidth 0                  userDefault
option add *FglBitmapMsgbox.relief      flat               userDefault
option add *FglButtonMsgbox.background  $config(defaultBg) userDefault
option add *FglButtonMsgbox.borderWidth 1                  userDefault
option add *FglButtonMsgbox.relief      raised             userDefault

################################################################################
## PROC winquestion: Create a Question box function
## Parameters :
##  mess        : Message in the window
##  title       : Title of window
##  options     : Options of the messagebox. Can be:
##                   text1, text2, text3,text4...   : Text of button
##                   defbuttonX  where X = {1,2,3}  : Default button
##                   dangbuttonX where X = {1,2,3}  : Dangereous button
##                   iconXXX
##                       where XXX = {question,information,stop,exclamation}
##  pid         : Pid of the calling program
##                   
## Returnings :
##  returnCode  : Number of button
################################################################################

proc winquestion { mess title options pid} {

# Globals
    global fglicon returnCode taskRoot
    global dataMsgbox
    global fglWtkClient dpyChannel


###################
# Analyse options #
###################
    set dc $dpyChannel
    set dataMsgbox($pid) $dc
    set dataMsgbox($dc,curr) 1
    set dangButtonMsgbox 0
    set icon ""
    set dataMsgbox($dc,text) ""

    foreach opt $options {
      switch -regexp $opt { \
	appmodal - \
	default_desktop_only - \
	iconhand - \
	iconasterisk - \
	setforeground - \
	systemmodal - \
	taskmodal { } \
	defbutton([0-9]+) { \
	  regexp {defbutton([0-9]+)} $opt dummy dataMsgbox($dc,curr) ;\
	} \
	dangbutton([0-9]+) { \
	  regexp {dangbutton([0-9]+)} $opt dummy dangButtonMsgbox ;\
	} \
	iconinformation { \
	  set icon "info.bmp" ;\
	  set iconreplace "info" ;\
	} \
	iconexclamation { \
	  set icon "exclam.bmp" ;\
	  set iconreplace "warning" ;\
	} \
	iconquestion { \
	  set icon "question.bmp" ;\
	  set iconreplace "question" ;\
	} \
	iconstop { \
	  set icon "stop.bmp" ;\
	  set iconreplace "error" ;\
	} \
	okcancel { \
	  set dataMsgbox($dc,text) { Ok Cancel } ;\
	} \
	retrycancel { \
	  set dataMsgbox($dc,text) { Retry Cancel } ;\
	} \
	yesno { \
	  set dataMsgbox($dc,text) { Yes No } ;\
	} \
	yesnocancel { \
	  set dataMsgbox($dc,text) { Yes No Cancel } ;\
	} \
	abortretryignore { \
	  set dataMsgbox($dc,text) { Abort Retry Ignore } ;\
	} \
	default { \
	  lappend dataMsgbox($dc,text) $opt ;\
	} \
      }
    }

    if { ![llength $dataMsgbox($dc,text)] } {
      set dataMsgbox($dc,text) Ok
    }

    if { ![string length $title] } {
      set title " "
    }

#####################
# Creation of frame #
#####################
    global taskRoot
    set w "$taskRoot.wq"
    set dataMsgbox($dc,win) $w
    catch {destroy $w}
    frame $w -class FglTopMsgbox
# don't delete 2 next lines. It's a wish bug 
    catch {destroy $w}
    frame $w -class FglTopMsgbox
    frame $w.ti -class FglTitleMsgbox
    label $w.ti.ti -class FglTitleMsgbox -text $title -borderwidth 0
##################
# Movable window #
##################
    if { [info commands fglMoveBind]=="fglMoveBind" } {
      fglMoveBind $w "$w.ti.ti $w.ti"
    }
    pack $w.ti -side top -fill both 
    pack $w.ti.ti -side left
 
################
# Create Frame #
################
    frame $w.top -class FglFrameMsgbox
    pack $w.top -side top -fill both 

    frame $w.bot -class FglFrameMsgbox
    pack $w.bot -side bottom -fill both

#################
# Create bitmap #
#################
    if { [string length $icon] } {
      if { [catch {set f [open $fglicon/$icon r] } ] } {
	set icon $iconreplace
      } else {
	close $f
	if { $fglWtkClient } {
	  set icon "@$fglicon/$icon"
	} else {
	  set icon "%$fglicon/$icon"
	}
      }
      if { $fglWtkClient } {
        global tkOrgButtons
        set old $tkOrgButtons
        set tkOrgButtons    0
        button $w.top.logo -class FglBitmapMsgbox -bitmap $icon -command {}
        set tkOrgButtons    $old
      } else {
        label $w.top.logo -class FglBitmapMsgbox -bitmap $icon
      }
      pack $w.top.logo -side left -padx 3m -pady 3m
    }

###################
# Create message  #
###################
    
   message $w.top.tex -class FglMessMsgbox -width 3i -text "$mess"
   pack $w.top.tex -side right -expand 1 -fill both -padx 3m -pady 3m

#######################################################
# Create a row of buttons at the bottom of the dialog #
#######################################################

    incr dataMsgbox($dc,curr) -1
    incr dangButtonMsgbox -1
    set dataMsgbox($dc,choice) -1
    set i 0
    foreach but $dataMsgbox($dc,text) {
      button $w.bot.b$i -class FglButtonMsgbox \
			-text $but 
      bind $w.bot.b$i <1> "fglSendKey $dc F[expr $i + 25]"
      if { $i == $dangButtonMsgbox  } {
	catch { $w.bot.b$i configure -cursor pirate }
      }
      bind $w.bot.b$i <Key-Escape> { }
      bind $w.bot.b$i <Any-KeyPress> { }
      bind $w.bot.b$i <Control-KeyPress> { }
      if { $i == 0 } {
	if { $i == $dataMsgbox($dc,curr) } {
	  frame $w.bot.f$i -class FglButtonMsgbox -relief sunken
	} else {
	  frame $w.bot.f$i -class FglButtonMsgbox -relief flat
	}
	raise $w.bot.b$i
	pack $w.bot.f$i -side left -expand 1 -padx 3m -pady 2m
	pack $w.bot.b$i -in $w.bot.f$i -side left  -padx 2m  -pady 2m\
						      -ipadx 2m -ipady 1m
      } else {
       frame $w.bot.f$i -class FglButtonMsgbox
       if { $i == $dataMsgbox($dc,curr) } {
	 $w.bot.f$i config -relief sunken
       } else {
	$w.bot.f$i config -relief flat
       }
       raise $w.bot.b$i
       pack $w.bot.f$i -side left -expand 1 -padx 3m -pady 2m
       pack $w.bot.b$i -in $w.bot.f$i -side left -expand 1 -padx 3m  \
						    -pady 3m\
						    -ipadx 2m -ipady 1m
      }
      incr i
    }
    set nbButton $i

# calculate position of the windows
    
    update
    set x [expr ([winfo width $taskRoot] -[winfo reqwidth $w] )/2]
    if { $x < 0 } { set x 10 }
    set y [expr ([winfo height $taskRoot] -[winfo reqheight $w] )/2]
    if { $y < 0 } { set y 10 }
    place $w -x $x -y $y

# Binding
  if { $nbButton > 1 } {
    bind $w <Key-Escape> { }
    bind $w <Any-KeyPress> { }
    bind $w <Control-KeyPress> { }
    bind $w <Tab> "	
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief flat
		   incr dataMsgbox($dc,curr)
		   if { \$dataMsgbox($dc,curr) >= $nbButton } {
		    set dataMsgbox($dc,curr) 0
		   }
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief sunken
		  " 
    bind $w <Left> "	
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief flat
		   incr dataMsgbox($dc,curr) -1
		   if { \$dataMsgbox($dc,curr) < 0 } {
		     set dataMsgbox($dc,curr) [expr $nbButton -1 ]
		   }
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief sunken
		  " 
    bind $w <Right> "	
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief flat
		   incr dataMsgbox($dc,curr)
		   if { \$dataMsgbox($dc,curr) >= $nbButton } {
		    set dataMsgbox($dc,curr) 0
		   }
		   $w.bot.f\$dataMsgbox($dc,curr) configure -relief sunken
		  " 
  }
  bind $w <Return> "$w.bot.b\$dataMsgbox($dc,curr) flash;
                    fglSendKey $dc F\[expr \$dataMsgbox($dc,curr) + 25\]"

  set i 0

  foreach but $dataMsgbox($dc,text) {
    set firstletter [string range $but 0 0]
    bind $w <KeyPress-[string tolower $firstletter]> "fglSendKey $dc $firstletter"
    bind $w <KeyPress-[string toupper $firstletter]> "fglSendKey $dc $firstletter"
    incr i
  }

#  after 100 raise $w
  myFocus $w

}

proc myMsgboxEnd { pid } {
 global dataMsgbox

 if { ! [info exist dataMsgbox($pid) ] } {
   return
 }
 set dc $dataMsgbox($pid)

 catch {destroy $dataMsgbox($dc,win)}
 unset dataMsgbox($dc,text)
 unset dataMsgbox($dc,curr)
 unset dataMsgbox($dc,win)
}

##############################
# Managment of Pulldown menu #
##############################
if { ![info exists tkOrgLabels] } {
  set tkOrgLabels 1
}

set mousePdw -1
set indPdw -1
set oldPdw -1
set oldmousePdw -1
set pdw(name) pdw
option add *itemPulldownDisabled          gray45              userDefault
option add *FglFramePulldown.background   $config(defaultBg)  userDefault
option add *FglFramePulldown.relief       raised              userDefault
option add *FglFramePulldown.borderWidth  2                   userDefault
option add *FglListboxPulldown.background  $config(defaultBg) userDefault
option add *FglListboxPulldown.relief      flat               userDefault
option add *FglListboxPulldown.borderWidth 0                  userDefault
option add *FglListboxPulldown.background  $config(bgEntryNormal) \
							      userDefault
#option add *FglListboxPulldown.highlightBackground $config(bgEntryHighlight)
option add *FglListboxPulldown.highlightBackground $config(bgEntryHighlight) \
							      userDefault
option add *FglCommentPulldown.background $config(defaultBg)  userDefault
option add *FglCommentPulldown.relief      raised             userDefault
option add *FglCommentPulldown.borderWidth 1                  userDefault
set pdw(disabled) [option get . itemPulldownDisabled  ""]
################################################################################
## PROC fglPulldownDestroy : Destroy a pulldown menu
## Parameters :
##  name  : Name of pulldown
## Returnings :
##  none
################################################################################

proc fglPulldownDestroy { {name {}} } {
  global taskRoot pdw

  set name "$taskRoot.$pdw(name)$name"
  catch {destroy $name}
  focus $taskRoot
}

################################################################################
## PROC fglPulldownCreate : Create a pulldown menu
## Parameters :
##  title    : Title of the menu
##  lines    : Lines of the menu:
##              { {message} sendkey underline disabled {comment}} ...}
##  x        : Position x
##  y        : Position y
##  name     : Name of pulldown
##  calcpos  : = 1 if Calculate position of Pulldown from button of barmenu
##               or from last Pulldown
##             = 0 no calculate: take position x,y
## Returnings :
##  none
################################################################################

proc fglPulldownCreate { title lines  {x 10} {y 5} {name {}} {calcpos 0} } {

  global taskRoot dx dy dyl indPdw mousePdw pdw tkOrgLabels topWnd lastMenuId
  global dpyChannel

  set shortname "$name"
  set name "$taskRoot.$pdw(name)$name"

  catch {destroy $name}

  focus $taskRoot
  set mousePdw -1
  set indPdw   -1
  set oldPdw   -1

  frame $name  -class FglFramePulldown

  if { [string range $title 0 0] == "@" } {
    set title [string range $title 1 end]
    bind $name <Any-Leave> "fglPulldownDestroy $shortname"
  }

  set lastButton "$topWnd.menu.m$lastMenuId"
  if { $calcpos >= 1 } {
    if { $shortname == 1 } {
      if { [winfo exist $lastButton] } {
#puts stderr "rootxLast [winfo rootx $lastButton]"
#puts stderr "rootxTask [winfo rootx $taskRoot]"
#puts stderr "rootyLast [winfo rooty $lastButton]"
#puts stderr "rootyTask [winfo rooty $taskRoot]"
#puts stderr "height    [winfo height $lastButton]"
	set x [expr [winfo rootx $lastButton] - [winfo rootx $taskRoot]]
	set y [expr [winfo rooty $lastButton] - [winfo rooty $taskRoot] \
	       + [winfo height $lastButton]]
      } else {
	set calcpos 0
      }
    } else {
      set oldname "$taskRoot.$pdw(name)[expr $shortname - 1]"
      set x [expr [winfo rootx $oldname] - [winfo rootx $taskRoot] \
	       + [winfo width $oldname]]
      set y [expr [winfo rooty $oldname] - [winfo rooty $taskRoot] \
	       + [expr $calcpos - 1] * $dyl]
    }
  }

  if { $calcpos == 0 } {
    set x [expr $dx * $x ]
    set y [expr $dy * $y ]
  }

  place $name -x $x \
	      -y $y

  listbox $name.l  -class FglListboxPulldown

  if { $title != "" } {
    set oldOrg $tkOrgLabels
    set tkOrgLabels 1
    label $name.title -text "$title" \
		      -relief raised \
		      -borderwidth 1
    set tkOrgLabels $oldOrg
    pack $name.title -fill x
    if { [ string length $title ] > 10 } {
      set lgmax [ string length $title ]
    }
    frame $name.line -borderwidth 0 \
		     -width [expr $lgmax * $dx] \
		     -height 10 
    pack $name.line 
  } {
    set lgmax 10
  }

  set nb 0
  foreach line $lines {
    set lg [llength $line]
    set text  [lindex $line 0]

    if { $lg >= 2 } {
      set key [lindex $line 1]
    } else {
      set key ""
    }
    if { $key == "" } {
      set key [string range $text 0 0]
    } 

    if { $lg >= 3 } {
      set under [lindex $line 2]
    } else {
      set under -1
    }
    set under [lindex $line 2]
    if { $under < 0 || $under > [string length $text] } {
      set under -1
    } else {
      incr under
    }

    if { $lg >= 4 } {
      set state [lindex $line 3]
    } else {
      set state 1
    }
    if { $lg >= 5 } {
     set pdw($name.l,$nb,comm) "[lindex $line 4]"
    }
    if { [ string length $text ] > $lgmax } {
      set lgmax [string length $text ]
    }
    set pdw($name.l,$nb,key) $key
    $name.l insert end " $text "
    if { $state == 1 || $state == "normal" } {
      set state normal
    } {
      set state disabled
      catch {$name.l recolor $nb $pdw(disabled)}
    }
    set pdw($name.l,$nb,state) $state
    catch {$name.l underline $nb $under}
    incr nb
  }

  incr lgmax 2
  $name.l configure -geometry "${lgmax}x${nb}"
    

  pack $name.l
  focus $name.l

#  set cc ";catch {fglTimeOff $dpyChannel}"
  set cc ""
  bind $name.l  <Escape>           "fglSendKey $dpyChannel Escape $cc"
  bind $name.l  <Control-KeyPress> "fglSendKey $dpyChannel Control-%K $cc"
  bind $name.l  <Any-KeyPress>     "fglSendKey $dpyChannel %K $cc"
  bind $name.l  <Up>               "fglSendKey $dpyChannel Up $cc"
  bind $name.l  <Down>             "fglSendKey $dpyChannel Down $cc"
  bind $name.l  <Any-Enter>        {
		  if { $pdw(%W,state) == "normal" } {
		    set curr [ %W nearest %y ]
		    if { $pdw(%W,$curr,state) == "normal" } {
		      %W highlight [ %W nearest %y ]
		    } else {
		      %W highlight -1
		    }
		  }
  }
  bind $name.l  <1>                "
		  set curr \[ %W nearest %y \]
		  if { \$pdw(%W,state) == \"normal\" \
		    &&  \$pdw(%W,\$curr,state) == \"normal\" } {
		    if { \[info exists pdw(%W,\$curr,key) \] } {
		      set indPdw \$curr
		      set mousePdw \$curr
		      fglSendKey $dpyChannel F19
		      fglSendKey $dpyChannel \$pdw(%W,\$curr,key)
                      $cc
		    }
		  }
  "
  bind $name.l  <Any-Motion>       "
		  if { \$pdw(%W,state) == \"normal\" } {
		    set i \[ %W nearest %y \] 
		    if { \$oldPdw != \$i && \[info exists pdw(%W,\$i,comm) \] } {
		      if { \$mousePdw == -1 } {
			fglSendKey $dpyChannel F20
                        $cc
		      }
		      set mousePdw \$i
		      if { \$pdw(%W,\$i,state) == \"normal\" } {
			%W highlight \$i
		      } else {
			%W highlight -1
		      }
		      fglPulldownComment $taskRoot \"\$pdw(%W,\$i,comm)\"
		      set oldPdw \$i
		    }
		  }
  "
  set pdw($name.l,state) normal
  set pdw($name.l,fg) [lindex [$name.l config -foreground] 4]

}

################################################################################
## PROC fglPulldownItem : Activate a item of a menu
## Parameters :
##  num      : Num of item
##  name     : Name of pulldown
## Returnings :
##  none
################################################################################

proc fglPulldownItem { num {name {}} } {

  global pdw taskRoot

  set name "$taskRoot.$pdw(name)$name.l"

  if { ![winfo exists $name ] } {
    return
  }

  catch { 
    raise $name
    if { $pdw($name,$num,state) == "normal" } {
      $name highlight $num
    } else {
      $name highlight -1
    }
  }

}

################################################################################
## PROC fglPulldownState : Put a state in pulldown menu
## Parameters :
##  state    : State of the pulldown
##  name     : Name of pulldown
## Returnings :
##  none
################################################################################

proc fglPulldownState { state {name {}} } {

  global taskRoot pdw 


  set name "$taskRoot.$pdw(name)$name.l"

  if { ![winfo exists $name ] } {
    return
  }

  if { $state == "disabled" } {
    set pdw($name,state) disabled
    $name config -foreground $pdw(disabled)
  } else {
    set pdw($name,state) normal
    $name config -foreground $pdw($name,fg)
  }
    
}

################################################################################
## PROC fglPulldownComment : Put the current comment
## Parameters :
##  troot: Current Task
##  m    : Text
## Returnings :
##  none
################################################################################

proc fglPulldownComment { troot m } { 

  global  taskY taskX

  if { ! [info exists taskX ] } {
    set sx [fglRes screen.size.x]
    set sy [fglRes screen.size.y]
  } {
    set sx $taskX
    set sy $taskY
  }
  set w $troot.commentPulldown

  if { [winfo exist $w ] } {
    $w configure -text "$m"
  } else  {
    label $w \
    -class FglCommentPulldown \
    -width $sx \
    -text "${m}" 
    place $w \
      -x [fglFormX 0] \
      -y [fglFormY [expr ($sy - 1 )]]
  }

}
